/*
   examples of libdspam implementation
   compile with: gcc -o example example.c libdspam.a -ldb-4.1 
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "libdspam.h"

int
main (int argc, char **argv)
{
  char buffer[1024];
  char *message = malloc (1);
  long len = 1;

  DSPAM_CTX *CTX;               /* DSPAM Context */
  struct _ds_spam_signature SIG;        /* Example signature */

  /* Performs any driver-specific startup functions
   * (such as initializing internal lock tables) */
  dspam_init_driver ();

  /* read in the message from stdin */
  message[0] = 0;
  while (fgets (buffer, sizeof (buffer), stdin) != NULL)
  {
    len += strlen (buffer);
    message = realloc (message, len);
    if (message == NULL)
    {
      fprintf (stderr, "out of memory!");
      exit (EXIT_FAILURE);
    }
    strcat (message, buffer);
  }

  /* MESSAGE PROCESSING */

  /* initialize dspam 
   * modes:
   * DSM_PROCESS: standard processing   (SH or IH ++ depending on result)
   * DSM_ADDSPAM: relearn this message as spam           (SH++, IH--)
   * DSM_FALSEPOSITIVE: relearn this message as innocent (SH--, IH++)
   * 
   * flags:
   * DSF_CHAINED:   use chained tokens for improved accuracy
   * DSF_SIGNATURE: If used in DSM_PROCESS mode, will generate a
   * signature for reversal.  If used in DSM_ADDSPAM or
   * DSM_FALSEPOSITIVE mode, will use the signature
   * provided in CTX->signature and assume *message is
   * NULL.
   * 
   * DSF_IGNOREHEADER:   Ignore the message header; useful if the message
   * was forwarded into the system.
   * 
   * flow: whenever a new message comes in, it should always be run through
   * dspam with DSM_PROCESS.  If, after the matter, the user wishes
   * to mark it as spam, it should be run through with DSM_ADDSPAM.
   * 
   * if an innocent message is marked as spam, dspam should be run
   * with DSM_FALSEPOSITIVE.  the tokens are then reversed and  
   * relearned twice as innocent. 
   * 
   * the original message OR a signature may be used to relearn the
   * message.  signature functionality is provided for applications that
   * will not hang onto messages while they are being worked with (for
   * example, the DSPAM agent or most non-mail client applications).
   */

  /* STANDARD INBOUND PROCESSING */
  /* DSF_SIGNATURE is specified to create an example signature.  This signature
   * can be stored in lieu of the actual message, for storage in non-mail
   * client programs (such as the DSPAM agent) */

  if (strstr (message, "Content-Transfer-Encoding: base64"))
    CTX = dspam_init ("./example.dict", DSM_PROCESS,
                      DSF_CHAINED | DSF_SIGNATURE | DSF_COPYBACK);
  else
    CTX = dspam_init ("./example.dict", DSM_PROCESS,
                      DSF_CHAINED | DSF_SIGNATURE);

  if (CTX == NULL)
  {
    fprintf (stderr, "dspam_init failed!\n");
    exit (EXIT_FAILURE);
  }

  /* call dspam */
  if (dspam_process (CTX, message) != 0)
  {
    fprintf (stderr, "dspam_process failed: %s\n", db_strerror (CTX->result));
    exit (EXIT_FAILURE);
  }

  /* print result */
  printf ("dspam probability result: %f\n", CTX->probability);
  if (CTX->result == DSR_ISSPAM)
    printf ("message IS spam\n");
  if (CTX->result == DSR_ISINNOCENT)
    printf ("message IS NOT spam\n");

  if (CTX->signature == NULL)
  {
    printf ("no signature provided\n");
  }
  else
  {
    SIG.data = malloc (CTX->signature->length);
    if (SIG.data != NULL)
      memcpy (SIG.data, CTX->signature->data, CTX->signature->length);
  }

  SIG.length = CTX->signature->length;

  /* destroy context */

  /* CTX->message is an in/out variable; should be destroyed whenever you're
   * finished using it, or if you're not going to use it */

  _ds_destroy_message (CTX->message);

  if (dspam_destroy (CTX) != 0)
  {
    fprintf (stderr, "dspam_destroy failed!");
    exit (EXIT_FAILURE);
  }

  /* SPAM REPORTING */
  /* Call everything just like before, only with DSM_ADDSPAM as mode.
   * This example uses the actual message instead of a signature.  See the
   * false positive reporting section for an example using a signature. */

  CTX = dspam_init ("./example.dict", DSM_ADDSPAM, DSF_CHAINED);
  if (CTX == NULL)
  {
    fprintf (stderr, "dspam_init failed!\n");
    exit (EXIT_FAILURE);
  }

  /* call dspam */
  if (dspam_process (CTX, message) != 0)
  {
    fprintf (stderr, "dspam_process failed: %s\n", db_strerror (CTX->result));
    exit (EXIT_FAILURE);
  }

  /* print result */
  printf ("dspam probability result: %f\n", CTX->probability);
  if (CTX->result == DSR_ISSPAM)
    printf ("message IS spam\n");
  if (CTX->result == DSR_ISINNOCENT)
    printf ("message IS NOT spam\n");

  /* destroy context */

  /* CTX->message is an in/out variable; should be destroyed whenever you're
   * finished using it, or if you're not going to use it */

  _ds_destroy_message (CTX->message);

  if (dspam_destroy (CTX) != 0)
  {
    fprintf (stderr, "dspam_destroy failed!");
    exit (EXIT_FAILURE);
  }

  /* FALSE POSITIVE REPORTING */
  /* Report message as a false positive
   * Here we can use the original message or a signature.  Signature is used here
   * for example purposes, and message is left NULL. */

  CTX = dspam_init ("./example.dict", DSM_FALSEPOSITIVE,
                    DSF_CHAINED | DSF_SIGNATURE);
  if (CTX == NULL)
  {
    fprintf (stderr, "dspam_init failed!\n");
    exit (EXIT_FAILURE);
  }

  CTX->signature = &SIG;

  /* call dspam */
  if (dspam_process (CTX, NULL) != 0)
  {
    fprintf (stderr, "dspam_process failed: %s\n", db_strerror (CTX->result));
    exit (EXIT_FAILURE);
  }

  /* print result */
  printf ("dspam probability result: %f\n", CTX->probability);
  if (CTX->result == DSR_ISSPAM)
    printf ("message IS spam\n");
  if (CTX->result == DSR_ISINNOCENT)
    printf ("message IS NOT spam\n");

  /* destroy context */

  /* CTX->message is an in/out variable; should be destroyed whenever you're
   * finished using it, or if you're not going to use it */

  _ds_destroy_message (CTX->message);

  if (dspam_destroy (CTX) != 0)
  {
    fprintf (stderr, "dspam_destroy failed!");
    exit (EXIT_FAILURE);
  }

  /* Performs any driver-specific shutdown functions */
  dspam_shutdown_driver ();

  exit (EXIT_SUCCESS);

}
